#include <windows.h>
#include <stdio.h>
#include <afxres.h>
#include <tchar.h>
#include "resource.h"

#define WM_MYCOMMAND          WM_APP + 100
#define WM_CHANGECONTLABEL    WM_APP + 101

#define COUNTOF(x)  (sizeof(x)/sizeof(x[0]))

/////////////////////////////////////////////////////////////////////////////
// command protocol definitions

enum Command {
	CI_UPDATELABEL,
	CI_TOGGLEFONT
};

// common header

typedef struct {
	UINT command;
} COMMAND_STRUCT, * PCOMMAND_STRUCT;

// command UPDATE_LABEL params

typedef struct UPDATELABEL_STRUCT
{
	COMMAND_STRUCT hdr;
	UINT           ctrlId;
	LPTSTR         label;
} UPDATELABEL_STRUCT, * PUPDATELABEL_STRUCT;

// command TOGGLEFONT params

typedef struct TOGGLEFONT_STRUCT
{
	COMMAND_STRUCT hdr;
	UINT ctrlId;
} TOGGLEFONT_STRUCT, * PTOGGLEFONT_STRUCT;

/////////////////////////////////////////////////////////////////////////////
// globals

HINSTANCE g_hInst = NULL;
HWND hwndCtrlDlg = NULL, hwndContDlg = NULL;

/////////////////////////////////////////////////////////////////////////////
// implementation

BOOL CALLBACK CtrlDlgProc(HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		switch (wp)
		{
		case IDC_BUTTON1:
			{
				TCHAR txt[4096] = {0};
				HWND hwndEdit = GetDlgItem(hDlg, IDC_EDIT1);
				GetWindowText(hwndEdit, txt, COUNTOF(txt));
				SendMessage(GetParent(hDlg), WM_CHANGECONTLABEL, _tcslen(txt), (LPARAM)txt);
			}
			break;
		}
		break;

	case WM_CLOSE:
		//MessageBox(hDlg, TEXT("Child dialog must not be closed"), TEXT("Error"), MB_OK | MB_ICONHAND);
		SendMessage(GetParent(hDlg), msg, wp, lp);
		break;
	}

	return FALSE;
}

BOOL CALLBACK ContDlgProc(HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_MYCOMMAND:
		{
			PCOMMAND_STRUCT pcs = (PCOMMAND_STRUCT)lp;

			switch (pcs->command)
			{
			case CI_UPDATELABEL:
				{
					PUPDATELABEL_STRUCT puls = (PUPDATELABEL_STRUCT)lp;
					HWND hwndLabel = GetDlgItem(hDlg, puls->ctrlId);
					if (hwndLabel)
						SetWindowText(hwndLabel, puls->label);
				}
				break;

			case CI_TOGGLEFONT:
				{
					PTOGGLEFONT_STRUCT tss = (PTOGGLEFONT_STRUCT)lp;
					HWND hwndLabel = GetDlgItem(hDlg, tss->ctrlId);
					if (hwndLabel)
					{
						static int selector = 0;
						int fontType = (selector++ % 2) ? DEFAULT_GUI_FONT : SYSTEM_FIXED_FONT;
						HGDIOBJ hFont = GetStockObject(fontType);
						SendMessage(hwndLabel, WM_SETFONT, (WPARAM)hFont, TRUE);
					}
				}
				break;
			}
		}
		break;

	case WM_CLOSE:
		//MessageBox(hDlg, TEXT("Child dialog must not be closed"), TEXT("Error"), MB_OK | MB_ICONHAND);
		SendMessage(GetParent(hDlg), msg, wp, lp);
		break;
	}

	return FALSE;
}

BOOL CALLBACK MainDlgProc(HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		{
			RECT rc = {0};

			// create CONTROL child
			hwndCtrlDlg = CreateDialog(g_hInst, MAKEINTRESOURCE(IDD_CTRL), hDlg, (DLGPROC)CtrlDlgProc);
			GetClientRect(hwndCtrlDlg, &rc);
			OffsetRect(&rc, 10, 10);
			MoveWindow(hwndCtrlDlg, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, FALSE);
			ShowWindow(hwndCtrlDlg, SW_SHOWNORMAL);
			UpdateWindow(hwndCtrlDlg);
	
			// create CONTAINER child
			GetClientRect(hDlg, &rc);
			int offset = rc.right / 2;

			hwndContDlg = CreateDialog(g_hInst, MAKEINTRESOURCE(IDD_CONT), hDlg, (DLGPROC)ContDlgProc);
			GetWindowRect(hwndContDlg, &rc);
			MapWindowPoints(GetDesktopWindow(), hDlg, (LPPOINT)&rc, 2);
			OffsetRect(&rc, offset + 10, 10);
			MoveWindow(hwndContDlg, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, FALSE);
			ShowWindow(hwndContDlg, SW_SHOWNORMAL);
			UpdateWindow(hwndContDlg);
		}
		return TRUE;

	case WM_CHANGECONTLABEL:
		{
			// make sure CONTROL exists
		    if (!IsWindow(hwndContDlg))
		    	break;

		    // route label text to CONTAINER
		    UPDATELABEL_STRUCT uls = {0};
		    uls.hdr.command = CI_UPDATELABEL;
		    uls.ctrlId  = IDC_LABEL;
		    uls.label   = (LPTSTR)lp;

			SendMessage(hwndContDlg, WM_MYCOMMAND, 0, (LPARAM)&uls);

			TOGGLEFONT_STRUCT tss = {0};
		    tss.hdr.command = CI_TOGGLEFONT;
		    tss.ctrlId  = IDC_LABEL;

			SendMessage(hwndContDlg, WM_MYCOMMAND, 0, (LPARAM)&tss);
		}
		break;

	case WM_CLOSE:
		DestroyWindow(hwndCtrlDlg);
		DestroyWindow(hwndContDlg);
		EndDialog(hDlg, 0);
		break;
	}

	return FALSE;
}

int WINAPI _tWinMain(HINSTANCE hInst, HINSTANCE hPrev, LPTSTR szCmd, INT nShow)
{
    g_hInst = hInst;
	DialogBox(hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)MainDlgProc);
	return 0;
}
